#!/bin/sh
#------------------------------------------------------------------------------
# /tmp/install.sh - installation file for mini_httpd services on EIS/FAIR
#
# Copyright (c) 2003-2010  Marcus Herleb, info(at)herleb(dot).de
# Copyright (c) 2011-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2003-05-02 mh
# Last Update:  $Id: install.sh 71883 2020-01-11 19:49:55Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version..
#------------------------------------------------------------------------------

# include eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib

#exec 2>/tmp/mini_httpd-install-sh-trace-$$.log
#set -x

module_name='mini_httpd'

# set variables
mini_httpd_path=/usr/local/htdocs
tarfile=/var/run/${module_name}.backup.tar.gz

ssldir=/usr/local/ssl
certdir=${ssldir}/certs
newcertdir=${ssldir}/newcerts
privdir=${ssldir}/private

certconf=/etc/httpd/cert.conf
cert_old="/etc/httpd/${module_name}.pem"
cert_pem=${certdir}/${module_name}.pem
cert_crt=${newcertdir}/${module_name}.crt
cert_key=${privdir}/${module_name}.key

myecho --pack ${module_name} --info "running install.sh ..."

# update shared library file
update_jedlib ${module_name}

# set platform specific parameters
# 04.04.2011 / jed - whole function taken from the certs package
oldssldir=/usr/local/ssl
certsdir=/var/certs
ssldir=${certsdir}/ssl

if [ -d ${oldssldir} -a ! -h ${oldssldir} ]
then
    # old certs directory exists and is not a symbolic link, go on ...
    if [ ! -d ${certsdir} ]
    then
        # new certs directory doesn't exist, go on ...
        myecho --pack ${module_name} "moving directory to new location: ${oldssldir} -> ${ssldir} ..."

        certs_arch=${module_name}_${EISDATE}.tgz

        # create directory
        if [ ! -d ${certsdir} ]
        then
            mkdir -p ${certsdir}
            chmod 755  ${certsdir}
            chown root ${certsdir}
            chgrp root ${certsdir}
        fi

        # create backup of certs files
        if [ ! -f ${certsdir}/${certs_arch} ]
        then
            cd ${oldssldir}/..
            tar czf ${certsdir}/${certs_arch} ssl/
        fi

        # move files to new location
        if [ -d ${oldssldir} -a ! -d ${ssldir} ]
        then
            mv ${oldssldir} ${ssldir}

            ln -s ${ssldir} ${oldssldir}
        fi
    else
        # destination directory exists, fix symbolic link ...
        if [ ! -d ${oldssldir}.${EISDATE}-${EISTIME} ]
        then
            myecho --pack ${module_name} --force-output --error "Saving erroneous directory to ${oldssldir}.${EISDATE}-${EISTIME} ..."
            myecho --pack ${module_name} --force-output --error "Please check its content and remove it afterwards!"
            mv ${oldssldir} ${oldssldir}.${EISDATE}-${EISTIME}

            myecho --pack ${module_name} --force-output "restoring symbolic link ${oldssldir} -> ${ssldir} ..."
            ln -s ${ssldir} ${oldssldir}
        fi
    fi
fi

# check and create directories
myecho --pack ${module_name} "creating/verifying directory structure ..."
for DNAME in certs certs/old crl csr newcerts private revoked web
do
    if [ ! -d ${ssldir}/${DNAME} ]
    then
        mkdir -p ${ssldir}/${DNAME}
    fi
done

# check symbolic link
myecho --pack ${module_name} "creating/verifying symbolic links ..."
for SNAME in /usr/local/ssl /etc/ssl
do
    if [ ! -h ${SNAME} ]
    then
        rm -fr ${SNAME}
    fi

    if [ ! -h ${SNAME} ]
    then
        ln -s ${ssldir} ${SNAME}
    fi
done

rm -f /etc/rc2.d/[KS]??${module_name}

ln -sf /etc/init.d/${module_name} /etc/rc2.d/K10${module_name}
ln -sf /etc/init.d/${module_name} /etc/rc2.d/S90${module_name}

# create htpasswd link
ln -sf /etc/httpd/htpasswd ${mini_httpd_path}/.htpasswd

myecho --pack ${module_name} "adding menu ..."
/var/install/bin/add-menu --menu "setup.services.menu" "setup.services.${module_name}.menu" "Mini_HTTPD"

# restoring configuration data files
if [ -f ${tarfile} ]
then
    myecho --pack ${module_name} "restoring configuration data ..."
    tar xzf ${tarfile} -C /
    rm -f ${tarfile}
else
    # insert fqdn hostname for certificate creation
    sed "s#\[FQDN\]#`hostname --fqdn`#" /etc/httpd/cert.conf.DEFAULT > /etc/httpd/cert.conf
    chmod 600 /etc/httpd/cert.conf
fi

# install/update package configuration file
silent=''
if called_from_install --pack "${module_name}"
then
    silent='--quiet'
fi

update=false
if install_config ${silent} ${module_name}
then
    update=true
fi

if [ -f ${cert_old} ]
then
    myecho --pack ${module_name} "moving ${module_name} ssl certificate to new location ..."
    if [ ! -f ${cert_key} ]
    then
        sed -n '/-BEGIN RSA/,/-END RSA/p' ${cert_old} > ${cert_key}
    else
        myecho --pack ${module_name} --warn "- file '${cert_key}' already exists and will therefore not be moved!"
    fi

    if [ ! -f ${cert_crt} ]
    then
        sed -n '/-BEGIN CERTIFICATE/,/-END CERTIFICATE/p' ${cert_old} > ${cert_crt}
    else
        myecho --pack ${module_name} --warn "- file '${cert_crt}' already exists and will therefore not be moved!"
    fi

    if [ ! -f ${cert_pem} ]
    then
        sed -n '/-BEGIN/,/-END/p' ${cert_old} > ${cert_pem}
    else
        myecho --pack ${module_name} --warn "- file '${cert_pem}' already exists and will therefore not be moved!"
    fi

    # move old certificate file to certs/old directory
    mv ${cert_old} ${certdir}/old/`basename "{${cert_old}" '.pem'`-${EISDATE}-${EISTIME}.pem
fi

if [ -f ${cert_pem} ]
then
    myecho --pack ${module_name} --info "ssl certificate already exists ..."
else
    myecho --pack ${module_name} --warn "creating new ssl certificate ..."

    a=no
    if [ -f /var/install/packages/certs ]
    then
        # installed certs package found
        _ask_tmpfile=`mktemp -t mini_httpd-ask.XXXXXXXXXX`
        /var/install/bin/ask "Should the certs package used to create the certificate" "yes" >${_ask_tmpfile}
        rc=${?}
        a=`cat ${_ask_tmpfile}`
        rm -f ${_ask_tmpfile}

        # if ask break, ask returned 255
        if [ ${rc} = 255 ]
        then
            a=no
        fi
    fi

    case ${a} in
        yes )
            # use certs package to create certificate
            if [ ! -f ${certdir}/ca.pem ]
            then
                myecho --pack ${module_name} --info "Creating CA for SSL ..."
                /var/install/bin/certs-create-tls-certs ca batch
            fi

            fqdn_hostname="`hostname --fqdn`"

            if [ ! -f ${certdir}/mini_httpd.pem ]
            then
                myecho --pack ${module_name} -n "Please enter '"
                myecho --pack ${module_name} -n --warn "${fqdn_hostname}"
                myecho --pack ${module_name} "' if you're asked for the COMMON NAME!"
                /var/install/bin/certs-create-tls-certs web batch alternate "${module_name}" "${fqdn_hostname}"
            fi
            ;;
        * )
            # use build-in routine to create certificate
            sslconf_sik=${OPENSSL_CONF}

            # set OpenSSL environment variable to prevent an error message
            export OPENSSL_CONF=${certconf}

            /usr/bin/openssl req -new -x509 -days 365 -nodes -batch \
                -config ${certconf} -out ${cert_crt} -keyout ${cert_key} 2>/dev/null

            # restore previous OpenSSL environment variable state
            if [ -z "${sslconf_sik}" ]
            then
                unset OPENSSL_CONF
            else
                export OPENSSL_CONF=${sslconf_sik}
            fi

            # create pem file which contains the key and the certificate
            if [ ! -s ${cert_crt} ]
            then
                myecho --pack ${module_name} --force-output --error "The required certificate file '${cert_crt}' doesn't exist."
            elif [ ! -s ${cert_key} ]
            then
                myecho --pack ${module_name} --force-output --error "The required key file '${cert_key}' doesn't exist."
            fi

            if [ -s ${cert_crt} ]
            then
                {
                    if [ -s ${cert_key} ]
                    then
                        cat ${cert_key}
                        echo
                    fi
                    sed -n '/-BEGIN/,/-END/p' ${cert_crt}
                } > ${cert_pem}
            fi
            ;;
    esac
fi

if [ -f ${cert_pem} ]
then
    chmod 0644 ${cert_pem}
fi
if [ -f ${cert_key} ]
then
    chmod 0644 ${cert_key}
fi

# create symbolic links to certificates
if [ -f ${certdir}/ca.pem ]
then
    ln -sf ${certdir}/ca.pem ${mini_httpd_path}/ca.pem
fi

if [ -f ${cert_pem} ]
then
    ln -sf ${cert_pem} ${mini_httpd_path}/${module_name}.pem
fi

myecho --pack ${module_name} "checking certificate settings ..."
/usr/bin/openssl x509 -subject -dates -fingerprint -noout -in ${cert_pem} >/dev/null

myecho --pack ${module_name} "adding additional mini_httpd menu entries ..."

# rename start.htm to start.html
if [ -f ${mini_httpd_path}/start.html ]
then
    grep -q "start.htm " /etc/httpd/menu

    if [ $? -eq 0 ]
    then
        cp /etc/httpd/menu /etc/httpd/menu.tmp
        sed 's#start.htm #start.html#' /etc/httpd/menu.tmp > /etc/httpd/menu
        rm -f /etc/httpd/menu.tmp
    fi
fi

for FNAME in support sr syslog
do
    if [ -f ${mini_httpd_path}/${FNAME}.html ]
    then
        grep -q "${FNAME}.cgi" /etc/httpd/menu

        if [ $? -eq 0 ]
        then
            cp /etc/httpd/menu /etc/httpd/menu.tmp
            sed "s#${FNAME}.cgi #${FNAME}.html#" /etc/httpd/menu.tmp > /etc/httpd/menu
            rm -f /etc/httpd/menu.tmp
        fi
    fi
done

# delete menu entries (if already exist)
for CGINAME in ./adminlog.cgi adminlog.cgi ./serverswitch.cgi serverswitch.cgi
do
    /usr/local/bin/httpd-menu.sh rem "${CGINAME}"
done

# make menu entry (to update old installations)
if [ -f ${mini_httpd_path}/adminlog.cgi ]
then
    /usr/local/bin/httpd-menu.sh add "adminlog.cgi" "Adminlog" "Main"
fi

if [ -f ${mini_httpd_path}/serverswitch.cgi ]
then
    /usr/local/bin/httpd-menu.sh add "serverswitch.cgi" "Serverswitch" "Tools"
fi

# configuration
myecho --pack ${module_name} "creating ${module_name} configuration ..."

# updated installation
/var/install/config.d/${module_name}.sh

if [ -f /etc/init.d/${module_name} ]
then
    /etc/init.d/${module_name} restart
fi

exit 0
