#!/bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/mini_httpdupdate.sh - configuration generator script
#     for /etc/config.d/mini_httpd and /etc/config.d/mini_httpd-access files.
#
# Copyright (c) 2003-2010  Marcus Herleb, info(at)herleb(dot).de
# Copyright (c) 2011-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2003-05-02 mh
# Last Update:  $Id: mini_httpd-update.sh 71883 2020-01-11 19:49:55Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version..
#------------------------------------------------------------------------------

# include eislib etc.
. /var/install/include/eislib
. /var/install/include/configlib
. /var/install/include/jedlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

#------------------------------------------------------------------------------
# function: variable is set
#
# input:  $1 - variable name
# return:  0 - variable set
#          1 - variable not set
#------------------------------------------------------------------------------
variable_set ()
{
    local _vs_var1
    local _vs_var2
    local _vs_ret

    eval _vs_var1=\$"${1}"
    eval _vs_var2=\$"{${1}+EMPTY}"

    if [ -z "${_vs_var1}" ]
    then
        if [ "${_vs_var2}" = 'EMPTY' ]
        then
            # variable is empty, but set
            _vs_ret=0
        else
            # variable is not set
            _vs_ret=1
        fi
    else
        # variable has a value
        _vs_ret=0
    fi

    return ${_vs_ret}
}

#------------------------------------------------------------------------------
# print variable only if it has been set
# $1 - variable name
# $2 - comment
#------------------------------------------------------------------------------
printsetvar ()
{
    if variable_set "${1}"
    then
        printvar "${1}" "${2}"
    fi
}

#------------------------------------------------------------------------------
# rename variables
#------------------------------------------------------------------------------
rename_variables ()
{
    local _rv_renamed
    local _rv_output_file
    local _rv_imax
    local _rv_idx
    local _rv_alias_old
    local _rv_alias_new

    _rv_renamed=0
    _rv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    # v1.2.10
    if [ ! -z "`grep ^MINI_HTTPD_START ${source_conf}`" ]
    then
        echo "- MINI_HTTPD_START -> START_MINI_HTTPD" >> ${_rv_output_file}
        START_MINI_HTTPD="${MINI_HTTPD_START}"
        _rv_renamed=1
    fi

    if [ ${MINI_HTTPD_USER_N} -eq 0 ]
    then
        _rv_imax=1
    else
        _rv_imax=${MINI_HTTPD_USER_N}
    fi

    _rv_idx=1
    while [ ${_rv_idx} -le ${_rv_imax} ]
    do
        if [ ! -z "`grep ^MINI_HTTPD_USER_${_rv_idx}_USERNAME ${source_conf}`" ]
        then
            eval _rv_alias_old='$MINI_HTTPD_USER_'${_rv_idx}'_USERNAME'
            eval _rv_alias_new="MINI_HTTPD_USER_${_rv_idx}_USER"

            echo "- MINI_HTTPD_USER_${_rv_idx}_USERNAME -> MINI_HTTPD_USER_${_rv_idx}_USER" >> ${_rv_output_file}
            eval "${_rv_alias_new}=\"${_rv_alias_old}\""
            _rv_renamed=1
        fi

        if [ ! -z "`grep ^MINI_HTTPD_USER_${_rv_idx}_PASSWORD ${source_conf}`" ]
        then
            eval _rv_alias_old='$MINI_HTTPD_USER_'${_rv_idx}'_PASSWORD'
            eval _rv_alias_new="MINI_HTTPD_USER_${_rv_idx}_PASS"

            echo "- MINI_HTTPD_USER_${_rv_idx}_PASSWORD -> MINI_HTTPD_USER_${_rv_idx}_PASS" >> ${_rv_output_file}
            eval "${_rv_alias_new}=\"${_rv_alias_old}\""
            _rv_renamed=1
        fi

        _rv_idx=`expr ${_rv_idx} + 1`
    done

    # v2.3.0
    if [ ! -z "`grep ^MINI_HTTPD_LOG_FILE ${source_conf}`" ]
    then
        if [ "${MINI_HTTPD_LOGFILE}" != 'none' ]
        then
            echo "- MINI_HTTPD_LOGFILE -> MINI_HTTPD_LOG_FILE" >> ${_rv_output_file}
            MINI_HTTPD_LOG_FILE="${MINI_HTTPD_LOGFILE}"
            _rv_renamed=1
        fi
    fi

    if [ ${_rv_renamed} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "renaming parameter(s) ..."
        cat ${_rv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for renamed parameter(s)!"
    fi

    rm -f ${_rv_output_file}
}

#------------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------------
modify_variables ()
{
    local _mv_modified
    local _mv_output_file

    _mv_modified=0
    _mv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_mv_modified} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "modifying parameter(s) ..."
        cat ${_mv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for modified parameter(s)!"
    fi

    rm -f ${_mv_output_file}
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
pre_add_variables ()
{
    local _pav_added
    local _pav_output_file

    _pav_added=0
    _pav_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    # v2.3.0
    if [ -z "`grep ^MINI_HTTPD_LOG= ${source_conf}`" ]
    then
        # read status from MINI_HTTPD_LOGFILE variable
        if [ "${MINI_HTTPD_LOGFILE}" = 'none' ]
        then
            echo "- MINI_HTTPD_LOG='no'" >> ${_pav_output_file}
            MINI_HTTPD_LOG='no'
        else
            echo "- MINI_HTTPD_LOG='yes'" >> ${_pav_output_file}
            MINI_HTTPD_LOG='yes'
        fi
        _pav_added=1
    fi

    if [ ${_pav_added} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "pre-adding new parameter(s) ..."
        cat ${_pav_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for modified parameter(s)!"
    fi

    rm -f ${_pav_output_file}
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
add_variables ()
{
    local _av_added
    local _av_output_file
    local _av_imax
    local _av_idx
    local _av_def_answer
    local _av_atest

    _av_added=0
    _av_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ -z "`grep ^MINI_HTTPD_BIND_HOST ${source_conf}`" ]
    then
        echo "- MINI_HTTPD_BIND_HOST='none'" >> ${_av_output_file}
        MINI_HTTPD_BIND_HOST='none'
        _av_added=1
    fi

    # v1.2.10
    if [ ${MINI_HTTPD_USER_N} -eq 0 ]
    then
        _av_imax=1
        _av_def_answer='no'
    else
        _av_imax=${MINI_HTTPD_USER_N}
        _av_def_answer='yes'
    fi

    _av_idx=1
    while [ ${_av_idx} -le ${_av_imax} ]
    do
        eval _av_atest='$MINI_HTTPD_USER_'${_av_idx}'_ACTIVE'

        if [ -z "${_av_atest}" -a -z "`grep ^MINI_HTTPD_USER_${_av_idx}_ACTIVE ${source_conf}`" ]
        then
            echo "- MINI_HTTPD_USER_${_av_idx}_ACTIVE='${_av_def_answer}'" >> ${_av_output_file}
            eval "MINI_HTTPD_USER_${_av_idx}_ACTIVE='${_av_def_answer}'"
            _av_added=1
        fi

        _av_idx=`expr ${_av_idx} + 1`
    done

    # v1.2.17
    if [ -z "`grep ^MINI_HTTPD_LOG_COUNT ${source_conf}`" ]
    then
        echo "- MINI_HTTPD_LOG_COUNT='10'" >> ${_av_output_file}
        MINI_HTTPD_LOG_COUNT='10'
        _av_added=1
    fi

    if [ -z "`grep ^MINI_HTTPD_LOG_INTERVAL ${source_conf}`" ]
    then
        echo "- MINI_HTTPD_LOG_INTERVAL='weekly'" >> ${_av_output_file}
        MINI_HTTPD_LOG_INTERVAL='weekly'
        _av_added=1
    fi

    if [ ${_av_added} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "adding new parameter(s) ..."
        cat ${_av_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for added parameter(s)!"
    fi

    rm -f ${_av_output_file}
}

#------------------------------------------------------------------------------
# delete variables
#------------------------------------------------------------------------------
delete_variables ()
{
    local _dv_deleted
    local _dv_output_file

    _dv_deleted=0
    _dv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_dv_deleted} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} "deleting old parameters ..."
        cat ${_dv_output_file}
    fi

    rm -f ${_dv_output_file}
}

#------------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------------
create_config ()
{
    local _cc_imax
    local _cc_idx

    myecho --pack ${module_name} "updating/creating configuration ..."

    {
        #------------------------------------------------------------------------------
        printgpl -conf "mini_httpd" "2011-04-04" "jed"
        #------------------------------------------------------------------------------

        if [ "${MINI_HTTPD_DO_DEBUG}" = "yes" -o "${MINI_HTTPD_DO_DEBUG}" = "no" ]
        then
            # debug active
            printvar "MINI_HTTPD_DO_DEBUG" "debug mode: yes or no"
            echo
        else
            # debug parameter exists but not active
            if [ -n "`grep MINI_HTTPD_DO_DEBUG ${source_conf}`" ]
            then
                echo "# MINI_HTTPD_DO_DEBUG='yes'                   # debug mode: yes or no"
                echo
            fi
        fi

        #------------------------------------------------------------------------------
        printgroup "general settings"
        #------------------------------------------------------------------------------
        printvar "START_MINI_HTTPD"      "activate configuration: yes or no"
        printvar "MINI_HTTPD_PORT"       "tcp port of the web server"
        printvar "MINI_HTTPD_HEADER"     "show header in browser: yes or no"
        printvar "MINI_HTTPD_SSL"        "use SSL: yes or no"
        printvar "MINI_HTTPD_BIND_HOST"  "hostname to bind to or none"

        #------------------------------------------------------------------------------
        printgroup "user settings"
        #------------------------------------------------------------------------------

        printvar "MINI_HTTPD_USER_N"    "number of users"

        if [ ${MINI_HTTPD_USER_N} -eq 0 ]
        then
            _cc_imax=1
        else
            _cc_imax=${MINI_HTTPD_USER_N}
        fi

        _cc_idx=1
        while [ ${_cc_idx} -le ${_cc_imax} ]
        do
            printvar "MINI_HTTPD_USER_${_cc_idx}_ACTIVE" "${_cc_idx}. activated: yes or no"
            printvar "MINI_HTTPD_USER_${_cc_idx}_USER"   "   name of user"
            printvar "MINI_HTTPD_USER_${_cc_idx}_PASS"   "   password"
            printvar "MINI_HTTPD_USER_${_cc_idx}_RIGHTS" "   rights"

            _cc_idx=`expr ${_cc_idx} + 1`
        done

        echo
        echo '#------------------------------------------------------------------------------'
        echo '# log handling'
        echo '#'
        echo '# Here you can specify how many logs should be saved and in with interval.'
        echo '# These parameter will only be used if the parameter MINI_HTTPD_LOGFILE'
        echo "# hasn't been set to 'none'"
        echo '#'
        echo '# Example:'
        echo "#   MINI_HTTPD_LOG='yes'"
        echo "#   MINI_HTTPD_LOGFILE='/var/log/mini_httpd.log'  - use this log file"
        echo "#   MINI_HTTPD_LOG_COUNT='10'                     - save the last 10 log files"
        echo "#   MINI_HTTPD_LOG_INTERVAL='daily'               - save one log file per day"
        echo '#------------------------------------------------------------------------------'
        printvar    "MINI_HTTPD_LOG"          "log server messages: yes or no"
        printsetvar "MINI_HTTPD_LOG_FILE"     "absolute pathe to log file"
        printvar    "MINI_HTTPD_LOG_COUNT"    "number of log files to save"
        printvar    "MINI_HTTPD_LOG_INTERVAL" "interval: daily, weekly, monthly"

        #------------------------------------------------------------------------------
        printend
        #------------------------------------------------------------------------------
    } > ${generate_conf}
}

#==============================================================================
# main
#==============================================================================

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

configfile=/etc/config.d/${module_name}
installfile=/var/run/${module_name}.install

# setting defaults
source_conf=${installfile}
generate_conf=${configfile}

goflag=0
modflag=0

case ${1} in
    *-update|update )
        goflag=1
        ;;
    *-test|test )
        source_conf=${configfile}
        generate_conf=/etc/config.d/${module_name}.test
        goflag=1
        ;;
    * )
        echo
        echo "Use one of the following options:"
        echo
        echo "  mini_httpd-update.sh [update] - the file ${configfile}.import will be read, the configuration will"
        echo "                                  be checked and an updated configuration file will be written."
        echo
        goflag=0
        ;;
esac

if [ ${goflag} -eq 1 ]
then
    . ${source_conf}

    pre_add_variables

    rename_variables
    modify_variables
    add_variables
    delete_variables

    create_config
fi

#==============================================================================
# end
#==============================================================================
exit 0
