#!/bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/mini_httpd.sh - configuration generator script
#
# Copyright (c) 2003-2010  Marcus Herleb, info(at)herleb(dot).de
# Copyright (c) 2011-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2003-05-02 mh
# Last Update:  $Id: mini_httpd.sh 71883 2020-01-11 19:49:55Z jed $
#
# Usage: mini_httpd.sh [--noconfirm][--quiet] [command]
#
#        --noconfirm               - don't show and confirm configlog screen
#        --quiet                   - don't show any screen output
#
# Available commands:
#
#            **without-additional-command**  - generates all configuration files
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version..
#------------------------------------------------------------------------------

# include eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

### other parameters ###
certdir=/usr/local/ssl/certs

mini_httpd_start_path=/etc/init.d
mini_httpd_path=/usr/local/htdocs
mini_httpd_file=/etc/config.d/${module_name}
mini_httpd_cert_file=/usr/local/ssl/certs/${module_name}.pem
mini_httpd_conf_file=/etc/httpd/${module_name}.conf
mini_httpd_log_file=/var/log/${module_name}.log
mini_httpd_pid_file=/var/run/${module_name}.pid
mini_httpd_menu_file=/var/install/menu/setup.services.${module_name}.menu
packagefile=/var/install/packages/${module_name}
mini_httpd_access_log_menu_name='View access log file'
generate_logrotate=/etc/logrotate.d/${module_name}

mini_httpd_version="v`grep "<version>" ${packagefile} | sed 's#<version>\(.*\)</version>#\1#'`"

### load configuration ###
. ${mini_httpd_file}
chmod 600 ${mini_httpd_file}

if [ "${MINI_HTTPD_LOG}" = 'yes' -a -z "${MINI_HTTPD_LOG_FILE}" ]
then
    # use default log file
    MINI_HTTPD_LOG_FILE="${mini_httpd_log_file}"
fi

#----------------------------------------------------------------------------------------
# creating certificate links
#----------------------------------------------------------------------------------------
create_cert_links ()
{
    # create symbolic links to certificates
    myecho --pack ${module_name} "creating certificate link ..."

    if [ -f ${certdir}/ca.pem ]
    then
        ln -sf ${certdir}/ca.pem ${mini_httpd_path}/ca.pem
    fi

    if [ -f ${mini_httpd_cert_file} ]
    then
        ln -sf ${mini_httpd_cert_file} ${mini_httpd_path}/${module_name}.pem
    fi
}

#----------------------------------------------------------------------------------------
# create mini_httpd configuration
#----------------------------------------------------------------------------------------
create_httpd_config ()
{
    myecho --pack ${module_name} "creating mini_httpd configuration ..."
    {
        # If mini_httpd is configured to do SSL/HTTPS, then the -S flag is available
        # to enable this feature.
        if [ "${MINI_HTTPD_SSL}" = 'yes' ]
        then
            echo "ssl"
        fi

        # Specifies a wildcard pattern for CGI programs, for instance "**.cgi" or
        # "cgi-bin/*". The default is no CGI.
        echo 'cgipat=**.cgi|**.php'

        # Specifies the character set to use with text MIME types.
        # The default is "iso-8859-1".
      # charset=iso-8859-1

        # Specifies a directory to chdir() to at startup. This is merely a convenience.
        # You could just as easily do a cd in the shell script that invokes the program.
        echo "dir=${mini_httpd_path}"

        # Specifies a hostname to bind to, for multihoming. The default is to bind to
        # all hostnames supported on the local machine.
        if [ "${MINI_HTTPD_BIND_HOST}" != 'none' ]
        then
            echo "host=${MINI_HTTPD_BIND_HOST}"
        fi

        # Specifies a log file name. The default is no logging.
        if [ "${MINI_HTTPD_LOG}" = 'yes' ]
        then
            if [ ! -f ${MINI_HTTPD_LOG_FILE} ]
            then
                touch ${MINI_HTTPD_LOG_FILE}
            fi

            if [ -f ${MINI_HTTPD_LOG_FILE} ]
            then
                chmod 0640 ${MINI_HTTPD_LOG_FILE}
                echo "logfile=${MINI_HTTPD_LOG_FILE}"
            fi
        fi

        # Do a chroot() at initialization time, restricting file access to the
        # program's current directory. The config-file option names for this flag
        # are 'chroot' and 'nochroot'
        echo 'nochroot'

        # Specifies a file to write the process-id to. If no file is specified, no
        # process-id is written. You can use this file to send signals to mini_httpd.
        echo "pidfile=${mini_httpd_pid_file}"

        # Specifies an alternate port number to listen on. The default is 80.
        echo "port=${MINI_HTTPD_PORT}"

        # If mini_httpd is configured to do SSL/HTTPS, then you can specify a server
        # certificate  with this flag.
        if [ "${MINI_HTTPD_SSL}" = 'yes' ]
        then
            echo "certfile=${mini_httpd_cert_file}"
        fi

        # Specifies what user to switch to after initialization when started as root.
        # The default is "nobody".
        echo 'user=root'
    } > ${mini_httpd_conf_file}
}

#----------------------------------------------------------------------------------------
# set file/directory access rights
#----------------------------------------------------------------------------------------
set_access_rights ()
{
    myecho --pack ${module_name} "setting file/directory access rights ..."

    chmod 0600 ${mini_httpd_conf_file}

    chmod -R 500 ${mini_httpd_path}/*.cgi
    chmod -R 400 ${mini_httpd_path}/*.css
    chmod -R 400 ${mini_httpd_path}/*.gif
    chmod -R 400 ${mini_httpd_path}/*.ico
    chmod -R 400 ${mini_httpd_path}/*.htm*
}

#----------------------------------------------------------------------------------------
# checking ssl port setting
#----------------------------------------------------------------------------------------
check_ssl_port ()
{
    local _csp_result
    local _csp_prog

    myecho --pack ${module_name} "checking ssl port setting ..."

    if [ "${MINI_HTTPD_SSL}" = "yes" ]
    then
      # _csp_result=`netstat --numeric-ports -lp | tr -s ' ' '#' | grep -v "mini_httpd#" | grep ":${MINI_HTTPD_PORT}#"`
        _csp_result=`ss -pnlt4 | tr -s ' ' '#' | cut -d'#' -f4,6 | grep -v '#users:(("mini_httpd"' | grep "^[0-9.]*:${MINI_HTTPD_PORT}#"`

        if [ $? -eq 0 ]
        then
            # active listener found

            # tcp#0#0#farragut.privatnet.l:22#0.0.0.0:*#LISTEN#2825/sshd#
            # ->                                                    ^^^^
          # _csp_prog=`echo "${_csp_result}" | cut -d'#' -f7 | cut -d'/' -f2`

            # 192.168.1.11:22#users:(("sshd",pid=2825,fd=4))
            # ->                       ^^^^
            _csp_prog=`echo "${_csp_result}" | head -n1 | sed 's/^.*#users:(("\([^"]*\).*$/\1/'`

            myecho --pack ${module_name} --warn "Current settings: MINI_HTTPD_SSL='yes'"
            myecho --pack ${module_name} --warn "                  MINI_HTTPD_PORT='${MINI_HTTPD_PORT}'"
            echo
            myecho --pack ${module_name} --warn "A program named '${_csp_prog}' is already listening on tcp-port '${MINI_HTTPD_PORT}'!"
            myecho --pack ${module_name} --warn "It might be necessary to change the tcp-port to non-used one!"
        fi
    else
        if [ "${MINI_HTTPD_PORT}" = "443" ]
        then
            myecho --pack ${module_name} --warn "Current settings: MINI_HTTPD_SSL='no'"
            myecho --pack ${module_name} --warn "                  MINI_HTTPD_PORT='${MINI_HTTPD_PORT}'"
            myecho --pack ${module_name} --warn "It might be necessary to change the tcp-port to a different value than '443'!"
            myecho --pack ${module_name} --warn "The tcp-port has been set to '443' although this port is usually"
            myecho --pack ${module_name} --warn "reserved for TLS usage. It is recommended to set the parameter to"
            myecho --pack ${module_name} --warn "different value, e.g. '8080'."
        fi
    fi
}

#----------------------------------------------------------------------------------------
# check access log menue entry
#----------------------------------------------------------------------------------------
check_access_menue_entry ()
{
    myecho --pack ${module_name} "checking access log menue entry ..."

    if [ "${MINI_HTTPD_LOG}" = 'yes' ]
    then
        # check if access log file menue entry exists
        grep -q "${mini_httpd_access_log_menu_name}" ${mini_httpd_menu_file}

        if [ $? -ne 0 ]
        then
            # entry doesn't exist, add it
            echo '<doc pre="pre-setup-services-mini_httpd-access-view-log" post="post-setup-services-mini_httpd-access-view-log" file="'${MINI_HTTPD_LOG_FILE}'">'${mini_httpd_access_log_menu_name}'</doc>' >> ${mini_httpd_menu_file}
        fi
    else
        # make sure that an access log file menue entry doesn't exist
        grep -q "${mini_httpd_access_log_menu_name}" ${mini_httpd_menu_file}

        if [ $? -eq 0 ]
        then
            # entry exists, remove it
            cp ${mini_httpd_menu_file} ${mini_httpd_menu_file}.tmp
            grep -v "${mini_httpd_access_log_menu_name}" ${mini_httpd_menu_file}.tmp > ${mini_httpd_menu_file}
            rm -f ${mini_httpd_menu_file}.tmp
        fi
    fi
}

#----------------------------------------------------------------------------------------
# processing user(s)
#----------------------------------------------------------------------------------------
process_users ()
{
    local _pu_idx
    local _pu_active_flag
    local _pu_active
    local _pu_user
    local _pu_pass
    local _pu_rights

    myecho --pack ${module_name} "processing user(s) ..."

    # delete previous files
    rm -f /etc/httpd/htpasswd /etc/httpd/rights

    if [ ${MINI_HTTPD_USER_N} -gt 0 ]
    then
        _pu_idx=1
        _pu_active_flag=0
        while [ ${_pu_idx} -le ${MINI_HTTPD_USER_N} ]
        do
            eval _pu_active='$MINI_HTTPD_USER_'${_pu_idx}'_ACTIVE'

            if [ "${_pu_active}" = "yes" ]
            then
                if [ ${_pu_active_flag} -eq 0 ]
                then
                    # create initial files
                    > /etc/httpd/htpasswd
                    > /etc/httpd/rights
                fi
                _pu_active_flag=1

                eval _pu_user='$MINI_HTTPD_USER_'${_pu_idx}'_USER'
                eval _pu_pass='$MINI_HTTPD_USER_'${_pu_idx}'_PASS'
                eval _pu_rights='$MINI_HTTPD_USER_'${_pu_idx}'_RIGHTS'

                myecho --pack ${module_name} "- ${_pu_user}"

                {
                    echo "${_pu_pass}"
                    echo "${_pu_pass}"
                } | /usr/sbin/htpasswd /etc/httpd/htpasswd ${_pu_user} > /dev/null

                echo "${_pu_user}=${_pu_rights}" >> /etc/httpd/rights
            fi

            _pu_idx=`expr ${_pu_idx} + 1`
        done

        if [ ${_pu_active_flag} -eq 0 ]
        then
            myecho --pack ${module_name} --warn "- no active users have been found!"
        fi
    else
        myecho --pack ${module_name} --error "- no users have been configured!"
    fi
}

#----------------------------------------------------------------------------------------
# create logrotate file
# input : $1 - username
#         $2 - groupname
#----------------------------------------------------------------------------------------
create_logrotate ()
{
    local _cl_uname
    local _cl_gname

    _cl_uname="$1"
    _cl_gname="$2"

    myecho --pack ${module_name} "creating logrotate configuration file ..."

    {
        #----------------------------------------------------------------------------------------
        print_short_header "${generate_logrotate}" "${pgmname}" "${mini_httpd_version}"
        #----------------------------------------------------------------------------------------

        if [ "${MINI_HTTPD_LOG}" = 'yes' ]
        then
            echo "${MINI_HTTPD_LOG_FILE} {"
            echo "    prerotate"
            echo "        ${mini_httpd_start_path}/${module_name} --quiet stop"
            echo "    endscript"

            echo "    rotate ${MINI_HTTPD_LOG_COUNT}"
            echo "    ${MINI_HTTPD_LOG_INTERVAL}"
            echo "    compress"
            echo "    notifempty"
            echo "    create 0640 ${_cl_uname} ${_cl_gname}"
            echo "    postrotate"
            echo "        ${mini_httpd_start_path}/${module_name} --quiet start"
            echo "    endscript"
            echo "    }"
            echo
        else
            echo "# Mini_HTTPD log creation disabled!"
        fi
    } > ${generate_logrotate}
}

#========================================================================================
# main
#========================================================================================

noconfirm=''
force_quiet_run=false
if [ $# -gt 0 ]
then
    # read parameter(s)
    while [ $# -gt 0 ]
    do
        case $1 in
            *-noconfirm )
                noconfirm=noconfirm
                shift
                ;;
            *-quiet )
                force_quiet_run=true
                shift
                ;;
            * )
                cmd="$1"
                break
                ;;
        esac
    done
fi

case ${cmd} in
    * )
        # generate all configuration files
        myecho --pack ${module_name} ""
        myecho --pack ${module_name} "${module_name} version: ${mini_httpd_version}"

        # generate passwords for all users
        if [ "${START_MINI_HTTPD}" = "yes" ]
        then
            myecho --pack ${module_name} "http port: ${MINI_HTTPD_PORT}"

            create_cert_links
            create_httpd_config
            create_logrotate 'root' 'root'
            process_users
            set_access_rights
            check_access_menue_entry
            check_ssl_port
        else
            myecho --pack ${module_name} --warn "package has been deactivated!"
        fi

        myecho --pack ${module_name} "finished."
        ;;
esac

#========================================================================================
# End
#========================================================================================
exit 0
